# -*- coding: utf-8 -*-
"""
Tests for the nudatus module.
"""
import builtins
import sys
import tokenize
from unittest import mock

import nudatus
import pytest


def test_get_version():
    """
    Ensure a call to get_version returns the expected string.
    """
    result = nudatus.get_version()
    assert result == ".".join([str(i) for i in nudatus._VERSION])


def test_mangle_script():
    """
    Check that mangle returns the expected output
    """
    script = ""
    real_mangled = b""
    with open("tests/bigscript.py") as f:
        script = f.read()
    assert len(script) > 0
    with open("tests/bigscript_mangled.py") as f:
        real_mangled = f.read()
    assert len(real_mangled) > 0
    mangled = nudatus.mangle(script)
    assert mangled == real_mangled


def test_mangle_with_bad_syntax():
    """
    Check that mangle throws an
    exception for a badly formatted script
    """
    script = ""
    with open("tests/bigscript_bad.py") as f:
        script = f.read()
    assert len(script) > 0
    with pytest.raises(tokenize.TokenError):
        nudatus.mangle(script)


def test_main_without_file(capfd):
    with mock.patch("sys.argv", ["nudatus"]):
        with pytest.raises(SystemExit) as ex:
            nudatus.main()
            assert ex.value.code == 1
            out, err = capfd.readouterr()
            assert len(out) == 0
            assert err == "No file specified"


def test_main_with_file_without_output_file(capfd):
    script = ""
    with open("tests/bigscript_mangled.py") as f:
        script = f.read()
    assert len(script) > 0
    with mock.patch("sys.argv", ["nudatus", "tests/bigscript.py"]):
        nudatus.main()
        out, err = capfd.readouterr()
        assert len(err) == 0
        assert out == script


def test_main_with_file_with_output_file(capfd):
    expected = ""
    with open("tests/bigscript_mangled.py") as f:
        expected = f.read()
    assert len(expected) > 0
    script = ""
    with open("tests/bigscript.py") as f:
        script = f.read()
    assert len(script) > 0
    with mock.patch("sys.argv", ["nudatus", "tests/bigscript.py", "testout.py"]):
        m = mock.mock_open(read_data=script)
        with mock.patch.object(builtins, "open", m):
            nudatus.main()
        m.assert_called_with("testout.py", "w")
        handle = m()
        handle.write.assert_called_with(expected)
        out, err = capfd.readouterr()
        assert len(err) == 0
        assert len(out) == 0


def test_main_with_bad_script(capfd):
    with pytest.raises(SystemExit) as ex:
        with mock.patch("sys.argv", ["nudatus", "tests/bigscript_bad.py"]):
            nudatus.main()
        assert ex.value.code == 1
    out, err = capfd.readouterr()
    assert len(out) == 0
    assert len(err) > 0
    assert err.startswith("Error mangling tests/bigscript_bad.py:")



# -*- coding: utf-8 -*-
"""
nudatus strips comments from Python scripts
https://pypi.org/project/nudatus/

© 2018-2021 Zander Brown

See the LICENSE file for more information, or visit:
https://opensource.org/licenses/MIT
"""

import argparse
import sys
import token
import tokenize
from io import BytesIO
from tokenize import tokenize as tokenizer
from typing import List, Optional

_VERSION = (
    0,
    0,
    5,
)


def get_version() -> str:
    return ".".join([str(i) for i in _VERSION])


def mangle(text: str) -> str:
    """
    Takes a script and mangles it

    TokenError is thrown when encountering bad syntax
    """

    text_bytes = text.encode("utf-8")

    # Wrap the input script as a byte stream
    buff = BytesIO(text_bytes)
    # Byte stream for the mangled script
    mangled = BytesIO()

    last_tok = token.INDENT
    last_line = -1
    last_col = 0
    last_line_text = ""
    open_list_dicts = 0

    # Build tokens from the script
    tokens = tokenizer(buff.readline)
    for t, text, (line_s, col_s), (line_e, col_e), line in tokens:
        # If this is a new line (except the very first)
        if line_s > last_line and last_line != -1:
            # Reset the column
            last_col = 0
            # If the last line ended in a '\' (continuation)
            if last_line_text.rstrip()[-1:] == "\\":
                # Recreate it
                mangled.write(b" \\\n")

        # We don't want to be calling the this multiple times
        striped = text.strip()

        # Tokens or characters for opening or closing a list/dict
        list_dict_open = [token.LSQB, token.LBRACE, "[", "{"]
        list_dict_close = [token.RSQB, token.RBRACE, "]", "}"]

        # If this is a list or dict
        if t in list_dict_open or striped in list_dict_open:
            # Increase the dict / list level
            open_list_dicts += 1
        elif t in list_dict_close or striped in list_dict_close:
            # Decrease the dict / list level
            open_list_dicts -= 1

        # Remove docstrings
        # Docstrings are strings not used in an expression,
        # unfortunatly it isn't as simple as "t is string and t
        # not in expression"
        if t == token.STRING and (
            last_tok == token.INDENT
            or (
                (
                    last_tok == token.NEWLINE
                    or last_tok == tokenize.NL
                    or last_tok == token.DEDENT
                    or last_tok == tokenize.ENCODING
                )
                and open_list_dicts == 0
            )
        ):
            # Output number of lines corresponding those in
            # the docstring comment
            mangled.write(b"\n" * (len(text.split("\n")) - 1))
        # Or is it a standard comment
        elif t == tokenize.COMMENT:
            # Plain comment, just don't write it
            pass
        else:
            # Recreate indentation, ideally we should use tabs
            if col_s > last_col:
                mangled.write(b" " * (col_s - last_col))
            # On Python 3 the first token specifies the encoding
            # but we already know it's utf-8 and writing it just
            # gives us an invalid script
            if t != tokenize.ENCODING:
                mangled.write(text.encode("utf-8"))

        # Store the previous state
        last_tok = t
        last_col = col_e
        last_line = line_e
        last_line_text = line

    # Return a string
    return mangled.getvalue().decode("utf-8")


def main(argv: Optional[List[str]] = None) -> None:
    """
    Command line entry point
    """
    if not argv:
        argv = sys.argv[1:]

    parser = argparse.ArgumentParser(
        description="Strip comments from a Python script.",
        epilog="nūdātus “strip, make naked”",
    )
    parser.add_argument("input", nargs="?", default=None)
    parser.add_argument("output", nargs="?", default=None)
    parser.add_argument(
        "--version", action="version", version="%(prog)s " + get_version()
    )
    args = parser.parse_args(argv)

    if not args.input:
        print("No file specified", file=sys.stderr)
        sys.exit(1)

    try:
        with open(args.input, "r") as f:
            res = mangle(f.read())
            if not args.output:
                print(res, end="")
            else:
                with open(args.output, "w") as o:
                    o.write(res)
    except Exception as ex:
        print("Error mangling {}: {!s}".format(args.input, ex), file=sys.stderr)
        sys.exit(1)


if __name__ == "__main__":  # pragma: no cover
    main(sys.argv[1:])



#!/usr/bin/env python3
from setuptools import setup
from nudatus import get_version


with open("README.rst") as f:
    readme = f.read()
with open("CHANGES.rst") as f:
    changes = f.read()


extras_require = {
    "tests": [
        "pytest>=4.6",
        "pytest-cov",
        "pytest-random-order>=1.0.0",
        "pytest-faulthandler",
        "coverage",
        "black",
    ],
    "package": [
        # Wheel building and PyPI uploading
        "wheel",
        "twine",
    ],
}

extras_require["dev"] = extras_require["tests"] + extras_require["package"]

extras_require["all"] = list(
    {req for extra, reqs in extras_require.items() for req in reqs}
)

setup(
    name="nudatus",
    version=get_version(),
    description="Strip comments from scripts, intended for use with "
    "MicroPython and other storage constrained "
    "environments",
    long_description=readme + "\n\n" + changes,
    author="Zander Brown",
    url="https://github.com/zanderbrown/nudatus",
    py_modules=[
        "nudatus",
    ],
    license="MIT",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Environment :: Console",
        "Intended Audience :: Developers",
        "License :: OSI Approved :: MIT License",
        "Operating System :: POSIX",
        "Operating System :: Microsoft :: Windows",
        "Programming Language :: Python :: 3.5",
    ],
    python_requires=">=3.5",
    extras_require=extras_require,
    entry_points={
        "console_scripts": ["nudatus=nudatus:main"],
    },
)



from microbit import *


















obj = {
    "key": "val"
}

arr = [
    'a',
    'b'
]

def show_num(num):
    if 1 + 1 == 2 and 2 + 2 == 4 or 3 + 3 == 6 \
       and 3 - 3 == 0 and 'this is nonsense' == 'this is nonsense':
        print('Hello')
        text = """
		This should survive
		"""
    i = Image()
    i.fill(0)
    if num & 0b00001000:
        i.set_pixel(0, 2, 9)
    if num & 0b00000100:
        i.set_pixel(1, 2, 9)
    if num & 0b00000010:
        i.set_pixel(2, 2, 9)
    if num & 0b00000001:
        i.set_pixel(3, 2, 9)
    display.show(i)

show_num(12)



from microbit import *

#Lorem ipsum dolor sit amet, consectetur adipiscing elit. Morbi vestibulum eu sapien commodo posuere. Nunc cursus justo non sem elementum, id finibus erat porta. Ut aliquam vestibulum urna ornare porta. Praesent laoreet iaculis erat, bibendum semper tellus blandit quis. Vivamus vitae tellus vel quam elementum porta. Morbi neque lorem, condimentum id blandit vel, ultrices at urna. Sed nec malesuada risus, nec tincidunt ante. Curabitur vel purus porttitor, hendrerit odio eu, dictum libero. Quisque non elit sodales, facilisis nisl vel, mollis mauris. Vestibulum placerat id diam id malesuada. Mauris nunc nisl, vehicula vel quam sed, finibus iaculis purus. Nunc at ullamcorper ex. Vivamus vel vestibulum dui. Proin luctus vehicula nisl. Aenean ultricies id quam nec consequat.
#Quisque consequat non metus a efficitur. Aliquam eu luctus nulla. Vestibulum pellentesque elit vitae quam condimentum suscipit. Nullam maximus porttitor gravida. Quisque imperdiet orci vitae urna volutpat, ac condimentum arcu tempus. Nullam feugiat feugiat magna, eu sagittis libero rutrum sit amet. Pellentesque ut augue turpis. Nullam sed massa eget velit placerat sagittis. Etiam euismod augue nibh, volutpat convallis enim porttitor ut. Nullam non mi sagittis lectus rhoncus ultricies. Nulla tempus tortor eu lectus mollis pellentesque. In porta convallis lorem, at interdum eros blandit at. Integer in nunc ut velit aliquam scelerisque. Duis lacinia, enim porttitor hendrerit viverra, nibh urna tempor enim, id vestibulum nisl nisl a purus. Ut id tincidunt sapien. Fusce non ante justo.
#Duis in neque tristique, sollicitudin lorem et, condimentum ligula. Nullam in euismod tortor. Ut sed purus tristique, maximus augue non, placerat dolor. Praesent tincidunt libero felis, convallis venenatis dui blandit quis. Mauris nulla odio, commodo in ex ut, bibendum congue sem. Curabitur sed enim lectus. Nam justo est, maximus et commodo ac, laoreet eu nunc. Donec interdum tortor maximus vestibulum fringilla. Aenean nec sapien imperdiet, elementum odio quis, pharetra urna. Nunc varius imperdiet eleifend. Sed eget aliquet quam.
#In porttitor diam tortor, ut aliquam neque rhoncus aliquet. Proin odio nisi, gravida et orci ac, dignissim efficitur ligula. Nulla facilisi. Proin ut libero dolor. Sed risus lectus, cursus in nibh id, convallis malesuada risus. Maecenas vitae gravida est, et condimentum ligula. In tortor felis, venenatis ac laoreet sit amet, posuere finibus ante. Nulla magna arcu, interdum id tortor et, fringilla vehicula nibh. Integer sed velit quis nibh hendrerit fermentum ac vitae tellus.
#Praesent augue erat, cursus non pellentesque nec, porta nec elit. Nam faucibus augue vitae augue aliquet mattis. Sed laoreet quam non aliquet fringilla. Maecenas dapibus vestibulum sem, vel rhoncus sem rhoncus eu. Nam facilisis ante mauris, eget rutrum risus mollis id. Fusce scelerisque nisi dapibus, ullamcorper magna ut, suscipit lorem. Donec posuere ante egestas, sagittis eros a, gravida est. Donec congue sit amet orci non mattis. Phasellus ac neque ac leo volutpat efficitur et ut ex. Donec sit amet neque quis risus accumsan volutpat vel ac elit. Nam interdum ligula ac dapibus dignissim. Aliquam luctus venenatis volutpat. Sed et mi ac nisi iaculis imperdiet. Nulla velit eros, sollicitudin sit amet porttitor a, tincidunt et nisl. Morbi eros ante, elementum vitae arcu nec, malesuada fringilla urna. Sed at tempor turpis.
#Praesent id lacinia leo. Duis tincidunt tortor id velit dictum, id vehicula velit blandit. Nunc lectus mi, condimentum non convallis ac, laoreet eget mauris. Ut efficitur nibh ac ex faucibus euismod. Nam ultrices justo sed magna egestas, id sollicitudin sem accumsan. Proin at ligula urna. Praesent at iaculis metus, ut posuere quam. Vestibulum eleifend massa tellus. Vivamus suscipit nisi in leo eleifend, vitae condimentum velit dictum. In eleifend purus at congue rutrum. Praesent dictum tortor et erat mollis, quis bibendum sem tincidunt. Proin vulputate pharetra nulla, in maximus nibh condimentum at. Ut ultricies ipsum nec arcu tempor, sed volutpat justo egestas.
#Donec interdum feugiat est quis laoreet. Donec sed justo non odio sollicitudin pulvinar hendrerit eu risus. Aliquam viverra dui scelerisque dui rhoncus, sit amet pharetra elit lobortis. Orci varius natoque penatibus et magnis dis parturient montes, nascetur ridiculus mus. Suspendisse accumsan, nibh at fringilla rutrum, sem lacus suscipit neque, eget auctor lectus mauris vitae sapien. Integer nec felis a tortor interdum tempor non id odio. Integer tellus ipsum, facilisis nec vulputate eu, mollis ut neque. Nullam eget risus ornare, maximus sem sit amet, pretium dolor. Cras venenatis ultricies sapien, ut malesuada libero imperdiet id. Morbi enim lectus, tincidunt sed lectus in, pulvinar faucibus ante. Cras et aliquam mauris. Fusce sodales dui eu augue malesuada lobortis. Suspendisse potenti. Quisque porttitor scelerisque blandit.
#Quisque volutpat augue nisi, sed elementum nibh porttitor ut. Phasellus tincidunt suscipit arcu at molestie. Mauris convallis feugiat erat. Praesent fringilla mauris velit, quis sagittis lorem tristique non. Duis eu magna odio. Nam nec ante nec nisl commodo interdum placerat et orci. In et urna et velit commodo vehicula. Cras sed nisi accumsan, scelerisque tellus condimentum, consectetur nisi.

"""
Donec at dictum dolor. Nam eu neque nisi. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Nunc vitae feugiat dolor. Phasellus maximus, lacus quis posuere ornare, quam nisi sollicitudin velit, vitae tristique elit velit quis nisl. Nullam non bibendum tortor. Vivamus semper et metus vel sagittis. Ut lacinia maximus est, luctus sollicitudin eros blandit non. Morbi finibus vulputate lacus, sed interdum lectus laoreet sed. Donec fringilla nec lectus eget blandit. Aenean a imperdiet odio, in eleifend metus. Pellentesque at imperdiet magna, quis suscipit ante. Pellentesque facilisis sagittis dictum. Nunc in enim in lorem porta viverra. Nam vestibulum finibus felis, vitae lobortis lorem vestibulum id.
Donec eu efficitur mauris. Pellentesque tincidunt id leo eu blandit. Ut ac consequat purus. Integer egestas leo ac convallis consectetur. Nulla posuere turpis eu turpis feugiat blandit. Aliquam ut congue ex. Vivamus dapibus tincidunt libero ac lacinia. Aenean arcu mi, faucibus dapibus nunc vel, tincidunt semper nulla. Etiam nec ipsum eu eros fermentum mollis a eu justo. Pellentesque pulvinar, neque nec sodales elementum, tellus nulla molestie sapien, sit amet venenatis urna mi eu tellus. Curabitur nec eleifend ante.
Vivamus sapien massa, aliquet vitae libero vel, gravida vestibulum risus. Ut ac nisi non augue hendrerit ornare. Class aptent taciti sociosqu ad litora torquent per conubia nostra, per inceptos himenaeos. Maecenas non sem interdum nibh posuere ullamcorper eu sagittis dolor. Vivamus posuere diam non augue feugiat, ac vulputate turpis condimentum. Sed pulvinar magna mauris, ac gravida tellus dignissim quis. Etiam ut placerat arcu.
Nunc in ipsum diam. Donec gravida, nibh vel pellentesque pulvinar, nunc leo ultrices libero, non porta diam tellus quis odio. Cras aliquet, enim vitae vulputate hendrerit, urna augue euismod odio, sit amet pretium nisl urna vel dolor. In tristique sapien mauris, ac vulputate magna ornare nec. Donec rhoncus magna in orci porta feugiat. Cras nibh turpis, consectetur in nunc in, varius pretium libero. Proin eget orci feugiat, auctor libero sit amet, imperdiet ex. Cras tincidunt dapibus nunc, at aliquet mi volutpat ut.
Aliquam erat volutpat. Cras vel laoreet ipsum, vitae cursus libero. Integer mollis pulvinar lorem, vel pretium nulla maximus id. Suspendisse sagittis tortor ligula, suscipit imperdiet tortor rutrum et. Ut turpis ante, pellentesque vitae nisl at, consectetur ornare velit. Phasellus nisi lacus, iaculis id feugiat sit amet, commodo ut ex. Vestibulum ante ipsum primis in faucibus orci luctus et ultrices posuere cubilia Curae; Integer facilisis dignissim mi metus. 
"""
)
def show_num(num):
    if 1 + 1 == 2 and 2 + 2 == 4 or 3 + 3 == 6 \
       and 3 - 3 == 0 and 'this is nonsense' == 'this is nonsense':
        print('Hello')
        text = """
		This should survive
		""" # This shouldn't
    i = Image()
    i.fill(0)
    if num & 0b00001000:
        i.set_pixel(0, 2, 9)
    if num & 0b00000100:
        i.set_pixel(1, 2, 9)
    if num & 0b00000010:
        i.set_pixel(2, 2, 9) # A comment on a line
    if num & 0b00000001:
        i.set_pixel(3, 2, 9)
    display.show(i)

show_num(12)
